@@ -6,5 +6,6 @@
 # @File    : __init__.py
 
 # from abess.linear import PdasLm, PdasLogistic, PdasPoisson, PdasCox, L0L2Lm, L0L2Logistic, L0L2Poisson, L0L2Cox, GroupPdasLm, GroupPdasLogistic, GroupPdasPoisson, GroupPdasCox, abessLogistic
-from abess.linear import abessLogistic, abessLm, abessCox, abessPoisson, abessMultigaussian, abessMultinomial
-from abess.gen_data import gen_data, gen_data_splicing
+from abess.linear import abessLogistic, abessLm, abessCox, abessPoisson, abessMultigaussian, abessMultinomial, abessGamma
+from abess.pca import abessPCA, abessRPCA
+from abess.datasets import make_glm_data, make_multivariate_glm_data
